/*
 *  dbdump by Davide Libenzi (GDBM Database Dump)
 *  Copyright (C) 2008  Davide Libenzi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Davide Libenzi <davidel@xmailserver.org>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <gdbm.h>


static int dbdmp_puts(unsigned char const *str, int size, FILE *fout);
static int dbdmp_dump(char const *dbfile, FILE *fout);
static void dbdmp_usage(char const *prg);



static int dbdmp_puts(unsigned char const *str, int size, FILE *fout) {
	int i;
	char cbuf[32];

	for (i = 0; i < size; i++) {
		if (str[i] == ';') {
			fputs("\\;", fout);
		} else if (str[i] == '\\') {
			fputs("\\\\", fout);
		} else if (isprint(str[i])) {
			fputc(str[i], fout);
		} else {
			sprintf(cbuf, "\\x%02x", str[i]);
			fputs(cbuf, fout);
		}
	}

	return 0;
}

static int dbdmp_dump(char const *dbfile, FILE *fout) {
	GDBM_FILE gdbf;
	datum dkey, ddata;

    	if ((gdbf = gdbm_open((char *) dbfile, 0, GDBM_READER, 00664, NULL)) == NULL) {
		fprintf(stderr, "unable to open DB file: file='%s'\n", dbfile);
		return -1;
	}
	ddata = gdbm_firstkey(gdbf);
	while (ddata.dptr != NULL) {
		dkey = ddata;
		ddata = gdbm_fetch(gdbf, dkey);
		if (ddata.dptr != NULL) {
			dbdmp_puts((unsigned char const *) dkey.dptr, dkey.dsize, fout);
			fputc(';', fout);
			dbdmp_puts((unsigned char const *) ddata.dptr, ddata.dsize, fout);
			fputc('\n', fout);
			free(ddata.dptr);
		}
		ddata = gdbm_nextkey(gdbf, dkey);
		free(dkey.dptr);
	}
	gdbm_close(gdbf);

	return 0;
}

static void dbdmp_usage(char const *prg) {

	fprintf(stderr,
		"use: %s --dbfile DBFIL [--output OFIL] [--help]\n", prg);
}

int main(int ac, char **av) {
	int i, res;
	char const *dbfile = NULL, *ofile = NULL;
	FILE *fout = stdout;

	for (i = 1; i < ac; i++) {
		if (!strcmp(av[i], "--dbfile")) {
			if (++i < ac)
				dbfile = av[i];
		} else if (!strcmp(av[i], "--output")) {
			if (++i < ac)
				ofile = av[i];
		} else {
			dbdmp_usage(av[0]);
			return 1;
		}
	}
	if (dbfile == NULL) {
		dbdmp_usage(av[0]);
		return 1;
	}
	if (ofile != NULL && (fout = fopen(ofile, "w")) == NULL) {
		perror(ofile);
		return 2;
	}
	res = dbdmp_dump(dbfile, fout);
	if (fout != stdout)
		fclose(fout);

	return res == 0 ? 0: 3;
}

